/*******************************************************************
 *
 *	Copyright (c) 1994-1999 Jetico, Inc., Finland
 *	All rights reserved.
 *
 *	File: kg_defs.h
 *	Revision: 
 *	Created:
 *	Description: declaration of error codes and flags
 *
 *******************************************************************/

#ifndef __KG_DEFS
#define __KG_DEFS

#define KGSHA_ID	2

#define	MAXIMUM_KEY_SIZE_BYTES		64
#define	MAXIMUM_DIGEST_SIZE_BYTES	32
#define MAXIMUM_NUMBER_OF_KEYS		8

#define PASSWORD_MINIMUM_LENGTH 6
#define PASSWORD_MAXIMUM_LENGTH 128

#define SEED_LENGTH    128
#define IVECTOR_LENGTH 8

/*******************************************************
 *
 *	Error codes
 *
 *******************************************************/

#define ERROR_NO				0x00000000
#define ERROR_INTERNAL_PROBLEM			0x00000001
#define ERROR_INVALID_ALGORITHM			0x00000002
#define ERROR_INCORRECT_CREATE_FLAG		0x00000003
#define ERROR_INCORRECT_FORMAT_VERSION		0x00000004
#define ERROR_INCORRECT_BLOCK_SIGNATURE		0x00000005
#define ERROR_DAMAGED_DATA_BLOCK		0x00000006
#define ERROR_CANCELLED_BY_USER			0x00000007
#define ERROR_USER_PASSWORDS_MISMATCH		0x00000008
#define ERROR_INCORRECT_PASSWORD		0x00000009
#define ERROR_NOT_ENOUGH_INIT_KEYS		0x0000000a
#define ERROR_INCORRECT_ENCR_ALGORITHM		0x0000000b
#define ERROR_INCORRECT_HASH_ALGORITHM		0x0000000c
#define ERROR_INCORRECT_DATA_BLOCK_SIZE		0x0000000d
#define ERROR_NOT_ENOUGH_SPACE_FOR_KEY		0x0000000e
#define ERROR_INCORRECT_POOL            	0x0000000f

/*******************************************************
 *
 *	Create flags that specify what the CreateKeyHandle()
 *	procedure has to do.
 *
 *******************************************************/

/* new container is created, one password 
 will be used for opening it
*/
#define CFLAG_CREATE_WITH_SINGLE_PASSWORD		0x00000001

/*
The CreateKeyHandle procedure gets Data block that
contains information allowing to verify password 
entered by user. If the password is correct, load 
encryption key and return Key Handle
*/

#define CFLAG_VERIFY_AND_LOAD_KEY				0x00000002
/*
User wishes to change his password. Ask him to enter 
old and new passwords, then correct Data block 
according to the new password
*/
#define CFLAG_CHANGE_PASSWORD					0x00000003


/* Miscelleneous definitions */
#define CFLAG_LEAST_AVAILABLE	CFLAG_CREATE_WITH_SINGLE_PASSWORD

#define CFLAG_MOST_AVAILABLE	CFLAG_CHANGE_PASSWORD

#define CORRECT_CFLAG( cFlag ) (	(cFlag >= CFLAG_LEAST_AVAILABLE) && (cFlag <= CFLAG_MOST_AVAILABLE) )


#endif /*__KG_DEFS */
