/*******************************************************************
 *	Copyright (c) 1994-1998 Jetico, Inc., Finland
 *	All rights reserved.
 *
 *	File:		driver/bc_dev.c
 *
 *	Description:	BestCrypt pseudo-device driver.
 *
 *	Scope:		BestCrypt pseudo-device driver
 *
 *	Platforms:	Linux
 * 
 *	Ideas:		Igor M. Arsenin
 *			Serge A. Frolov
 *			Vitaliy G. Zolotarev
 *
 *	Author:		Nail R. Kaipov
 *
 *	Created:	10-Nov-1998
 *
 *	Revision:	6-Aug-1999.	new strategy routine
 *			15-Sep-1999.	2.3.x kernels support
 *
 *******************************************************************/

#ifndef EXPORT_SYMTAB
#define EXPORT_SYMTAB
#endif

#include "bc_cfg.h"
#include <linux/module.h>
#include <linux/sched.h>		/* inode et al            */
#include <linux/file.h>			/* inode et al            */
#include <linux/fs.h>			/* inode et al            */
#include <linux/proc_fs.h>		/* inode et al            */

#include <linux/stat.h>
#include <linux/vmalloc.h>		/* vmalloc()              */
#include <linux/kernel.h>		/* printk()               */
#include <asm/uaccess.h>		/* put_user()             */
#include <linux/hdreg.h>		/* hd_geometry            */

#include "bc_types.h"			/* bc types               */
#include "bc_ioctl.h"			/*           ioctl codes  */
#include "bc_mgr.h"
#include "bc_dev.h"			/* bc types        codes  */

#define DEVICE_OFF(dev)
#include <linux/blk.h>

#define __KERNEL_SYSCALLS__
#include <linux/unistd.h>

static struct bc_device	bc_dev		[MAX_BC_DEVS];
static int		bc_sizes	[MAX_MINORS];
static int		bc_blksizes	[MAX_MINORS];

static ssize_t bc_read   (struct file *, char *, size_t, loff_t *);
static ssize_t bc_write  (struct file *, const char *, size_t, loff_t *);
static int     bc_ioctl  (struct inode *, struct file *, unsigned int, unsigned long);
static int     bc_open   (struct inode *, struct file *);	
static int     bc_release(struct inode *, struct file *);

static void dummy_request_fn(void) { return; }
static int dummy_ioctl  (struct inode *i, struct file *f, u_int cmd, u_long arg) { return -ENODEV; }
static int dummy_open   (struct inode *i, struct file *f) { return -ENODEV; }
static int dummy_release(struct inode *i, struct file *f) { return -ENODEV; }

static struct file_operations save_fops = {
	read   :block_read,
	write  :block_write,
	ioctl  :dummy_ioctl,
	open   :dummy_open,
	release:dummy_release
};

static void (*save_request_fn)(void) = &dummy_request_fn;
static int *save_blksize;
static int *save_blksize_size;
static int max_loop;

static struct file_operations bc_fops = {
	read   :bc_read,
	write  :bc_write,
	ioctl  :bc_ioctl,
	open   :bc_open,
	release:bc_release
};


long	rqw_add = 0;
long	rqw_end = 0;
long	rqr_add = 0;
long	rqr_end = 0;
/*-----------------------------------------------------------*/
static inline int bc_minor(kdev_t dev)
{
	return ((MINOR(dev) >= BC_FIRST) && (MINOR(dev) <= BC_LAST));
}

static inline int bc_major(kdev_t dev)
{
	return (MAJOR_NR == MAJOR(dev));
}

static inline struct bc_device *get_bcdev(kdev_t dev)
{
	if (bc_minor(dev))
		return (&bc_dev[MINOR(dev)-BC_FIRST]);
	else
		return NULL;
}

/*-----------------------------------------------------------*/
struct request		*BC_CURRENT = NULL;

static void grab_requests();
static void bc_request();

static void do_bc_request() 
{
	if (NULL == CURRENT)
		return;
	grab_requests();
	save_request_fn();
	bc_request();
}


static void grab_requests()
{
	register struct request *prev, *tail, *curr;
	
	if (NULL == (curr = CURRENT))
		return;
	if (NULL != (tail = BC_CURRENT))
		while (tail->next)
			tail = tail->next;
	prev = NULL;
	while (curr) {
		if (bc_minor(curr->rq_dev)) {
			if (tail) tail->next = curr;
			else      BC_CURRENT = curr;
			if (prev) prev->next = curr->next;
			else      CURRENT    = curr->next; 
			tail = curr;
		} else {
			if (prev) prev = prev->next;
			else      prev = CURRENT; 
		}
		curr = curr->next;
	}
	if (tail) tail->next = NULL;
}

static void bc_request()
{
	struct bc_device	*bc;
	struct request		*req;
	struct inode		*inode;	
	struct file		*file;
	struct file_operations	*fops;
	size_t			len, ret;
	loff_t			pos;
	char			*dst;
	u16			iv[4];
	mm_segment_t		fs;
	int			uptodate;

repeat:
	if (!BC_CURRENT) return;
	req = BC_CURRENT;
	BC_CURRENT = req->next;
	uptodate = 0;
	bc = get_bcdev(req->rq_dev);
	if (!bc || !bc->bc_dentry || !bc->bc_dentry->d_inode) {
		goto error_out;
	} else if (WRITE == req->cmd) {
		if (bc->bc_flags & BC_FLAGS_READ_ONLY) {
			goto error_out;
		}
	} else if (READ != req->cmd) {
		printk("bc: unknown command (%d).", req->cmd);
		goto error_out;
	}
	if (!bc->bc_dentry->d_inode->i_op) {
		printk("null i_op\n");
		goto error_out;
	}
	if (!bc->bc_dentry->d_inode->i_op->default_file_ops) {
		printk("null default_fops\n");
		goto error_out;
	}
	fops  = bc->bc_dentry->d_inode->i_op->default_file_ops;
	pos   = req->sector * 512 + bc->bc_offset;
	len   = req->current_nr_sectors << 9;
	dst   = req->buffer;
	file  = bc->bc_file;
	inode = file->f_dentry->d_inode;
	 
	iv[0] = iv[1] = iv[2] = iv[3] = req->sector & 0xFFFF;

	if (READ == req->cmd) {
		fs = get_fs();
		set_fs(KERNEL_DS);
		spin_unlock_irq(&io_request_lock);
		ret = fops->read(file, dst, len, &pos);
		spin_lock_irq(&io_request_lock);
		set_fs(fs);
		if (ret != len) {
			printk("bc: read error wanted %d, read %d\n", len, ret);
			goto error_out;
		}
		bc->bc_alg->decrypt(bc->bc_key, (char *)iv, dst, dst, len);
	} else {
		bc->bc_alg->encrypt(bc->bc_key, (char *)iv, dst, dst, len);
		fs = get_fs();
		set_fs(KERNEL_DS);
		spin_unlock_irq(&io_request_lock);
		ret = fops->write(file, dst, len, &pos);
		spin_lock_irq(&io_request_lock);
		set_fs(fs);
		bc->bc_alg->decrypt(bc->bc_key, (char *)iv, dst, dst, len);
		if (ret != len) {
			printk("bc: write error wanted %d, read %d\n", len, ret);
			goto error_out;
		}
	}
	uptodate = 1;
error_out:	
        if (!end_that_request_first(req, uptodate, DEVICE_NAME))
		end_that_request_last(req);
	
	goto repeat;
	

}

/*-- end of strategy routine --------------------------------*/



/*-- fops functions -----------------------------------------*/
static int bc_open(struct inode *inode, struct file *file)
{
	struct bc_device *bc;

	if (!inode)	return -EINVAL;
	if (!bc_major(inode->i_rdev)) {
		printk("bc_open: invalid major (%d).\n", MAJOR_NR);
		return -ENODEV;
	}
	if (!bc_minor(inode->i_rdev))
		return save_fops.open(inode, file);
	else
		bc = get_bcdev(inode->i_rdev);
	bc->bc_refcnt++;
	MOD_INC_USE_COUNT;
	return 0;
}

static int bc_release(struct inode *inode, struct file *file)
{
	struct bc_device *bc;
	int error;
	
	if (!inode)	
		return 0;
	if (!bc_major(inode->i_rdev)) {
		printk("bc_release: invalid major (%d).\n", MAJOR_NR);
		return 0;
	}
	if (!bc_minor(inode->i_rdev))
		return save_fops.release(inode, file);
	else
		bc = get_bcdev(inode->i_rdev);

	error = fsync_dev(inode->i_rdev);

	if (bc->bc_refcnt <= 0)	
		printk("bc_release: refcount (%d) <= 0\n", bc->bc_refcnt);
	else  {
		bc->bc_refcnt--;
		MOD_DEC_USE_COUNT;
	}
	return error;
}

static ssize_t bc_read (struct file *file, char *buf, size_t count, loff_t *ppos)
{
	struct bc_device *bc;
	struct inode	*inode;
	
	if (!file)	
		return -EINVAL;
	
	inode = file->f_dentry->d_inode;	

	if (!bc_major(inode->i_rdev)) {
		printk("bc_read: invalid major (%d).\n", MAJOR_NR);
		return -ENODEV;
	}
	if (NULL == (bc = get_bcdev(inode->i_rdev)))
		return save_fops.read(file, buf, count, ppos);

//!--	if (bc->bc_flags & BC_FLAGS_DEV_LOCK)
//!--		return -EIO;

	return block_read(file, buf, count, ppos);
}

static ssize_t bc_write(struct file *file, const char *buf, size_t count, loff_t *ppos)
{
	struct bc_device *bc;
	struct inode	*inode;
		
	if (!file)
		return -EINVAL;
		
	inode = file->f_dentry->d_inode;
	
	if (!bc_major(inode->i_rdev)) {
		printk("bc_read: invalid major (%d).\n", MAJOR_NR);
		return -ENODEV;
	}
	if (NULL == (bc = get_bcdev(inode->i_rdev)))
		return save_fops.write(file, buf, count, ppos);

//!--	if (bc->bc_flags & BC_FLAGS_DEV_LOCK)
//!--		return -EIO;

	return block_write(file, buf, count, ppos);
}

/*-- end of fops functions ----------------------------------*/

/*-- ioctl functions ----------------------------------------*/
static int bc_get_info(struct bc_device *bc, struct bc_info *arg);
static int bc_set_fd(struct bc_device *bc, kdev_t dev, struct bc_file *arg);
static int bc_clr_fd(struct bc_device *bc, kdev_t dev);
static int bc_lock_dev(struct bc_device *bc, int lock);
static int bc_verify_alg(struct bc_alg *arg);
static int bc_make_key(struct bc_key *arg);
static int bc_free_key(struct bc_key *arg);
static int bc_process_block(struct bc_block *arg, int operation);

/*-----------------------------------------------------------*/
static int bc_ioctl(struct inode *inode, struct file *file, u_int cmd, u_long arg)
{
	struct bc_device *bc;
	struct hd_geometry *geo;
	unsigned long tmp;
	int err;
	
	if (!inode) 
		return -EINVAL;
	if (!bc_major(inode->i_rdev))
		return -ENODEV;
		
        if (!bc_minor(inode->i_rdev)) {
		err = save_fops.ioctl(inode, file, cmd, arg);
		if (max_loop) {
			if (save_blksize)
				memcpy(bc_sizes, save_blksize, (max_loop)*sizeof(int));
			if (save_blksize_size)
				memcpy(bc_blksizes, save_blksize_size, (max_loop)*sizeof(int));
		}
		return err;
	} else
		bc = get_bcdev(inode->i_rdev);

	switch (cmd) {
	case BC_GET_INFO:
		return bc_get_info(bc, (struct bc_info *) arg);
	case BC_SET_FD:
		return bc_set_fd(bc, inode->i_rdev, (struct bc_file *) arg);
	case BC_CLR_FD:
		return bc_clr_fd(bc, inode->i_rdev);
	case BC_LOCK_DEV:
		return bc_lock_dev(bc, 1);
	case BC_UNLOCK_DEV:
		return bc_lock_dev(bc, 0);
	case BC_VERIFY_ALG:
		return bc_verify_alg((struct bc_alg*) arg);
	case BC_MAKE_KEY:
		return bc_make_key((struct bc_key*) arg);
	case BC_FREE_KEY:
		return bc_free_key((struct bc_key*) arg);
	case BC_ENCRYPT_BLOCK:
		return bc_process_block((struct bc_block*) arg, BC_ENCRYPT_BLOCK);
	case BC_DECRYPT_BLOCK:
		return bc_process_block((struct bc_block*) arg, BC_DECRYPT_BLOCK);
/* Block device specific */
	case BLKGETSIZE:   /* Return device size */
		if (!bc->bc_dentry)	
			return -ENXIO;
		if (!arg)
			return -EINVAL;
		put_user(bc_sizes[bc->bc_number] << 1, (long *)arg);
		return 0;
	case HDIO_GETGEO:	/* return pseudo geometry for mkfs.msdos */
		if (!bc->bc_dentry)
			return -ENXIO;
		if (!arg)
			return -EINVAL;
		geo = (struct hd_geometry *) arg;

		tmp = 0x80*0x80*0x8000 - bc_sizes[bc->bc_number];
		put_user (0x80, (u8 *)&geo->heads);
		put_user (0x80, (u8 *)&geo->sectors);
		put_user (0x8000, (u16 *)&geo->cylinders);
		put_user (tmp, (u32 *)&geo->start);
		return 0;
	default:
		return -EINVAL;
	}
	return 0;
}
/*-----------------------------------------------------------*/

static int bc_get_info(struct bc_device *bc, struct bc_info *arg)
{
	struct bc_info	query;
	long ver_major, ver_minor, dev_busy;
	 
	if (!arg)
		return -EINVAL;
	if (copy_from_user(&query, arg, sizeof(query)))
		return -EFAULT;

	ver_major = 0;
	ver_minor = 2;
	dev_busy  = bc->bc_flags & BC_FLAGS_DEV_BUSY ? 1 : 0;

	put_user(ver_major, (long *)&arg->ver_major);
	put_user(ver_minor, (long *)&arg->ver_minor);
	put_user(dev_busy,  (long *)&arg->busy);
	return 0;
}


#define MAX_DISK_SIZE 1024*1024*1024

static inline void figure_bc_size(struct bc_device *bc)
{
	int	size;

	if (S_ISREG(bc->bc_dentry->d_inode->i_mode)) {
		size = (bc->bc_dentry->d_inode->i_size - bc->bc_offset) / BLOCK_SIZE;
	} else {	/* special inode */
		kdev_t bcdev = bc->bc_dev;
		if (blk_size[MAJOR(bcdev)])
			size = blk_size[MAJOR(bcdev)][MINOR(bcdev)] - bc->bc_offset/BLOCK_SIZE;
		else
			size = MAX_DISK_SIZE;
	}
	bc_sizes[bc->bc_number] = size;
}


static int bc_set_fd(struct bc_device *bc, kdev_t dev, struct bc_file *arg)
/* key must be ready */
{
	struct file	*file;
	struct inode	*inode;
	struct bc_algorithm	*bc_alg;
	struct bc_file	query; 
	
	if (!arg)
		return -EINVAL;
	if (copy_from_user(&query, arg, sizeof(query)))
		return -EFAULT;
	
	if (bc->bc_dentry)
		return -EBUSY;
		
	bc_alg = get_bc_algo(query.alg_id, NULL);
	if (!bc_alg) 
		return -EINVAL;
	/* Check if key is valid */
	if (bc_alg->test_key(query.key_handle) != 0) 
		return -EINVAL;
	/* lock that key. always succeeds */
	if (bc_alg->lock_key(query.key_handle, 1) != 0) 
		return -EINVAL; 

	if (!(file = fget(query.fd)))
		return -EBADF;
	inode = file->f_dentry->d_inode;
	if (!inode) {
		printk("bc_set_fd: NULL inode.\n");
		fput(file);
		return -EINVAL;
	}

	if (S_ISBLK(inode->i_mode)) {
		int error = blkdev_open(inode, file);
		if (error) {   
			fput(file);
			return error;
		}
		bc->bc_dev   = inode->i_rdev;
		bc->bc_flags = 0;
	} else if (S_ISREG(inode->i_mode)) {
		bc->bc_dev   = inode->i_dev;
		bc->bc_flags = BC_FLAGS_DO_BMAP;
	} else {
		fput(file);
		return -EINVAL;
	}
	if (IS_RDONLY (inode) || is_read_only(bc->bc_dev)) {
		bc->bc_flags |= BC_FLAGS_READ_ONLY;
		set_device_ro(dev, 1);
	} else {
		invalidate_inode_pages (inode);
		set_device_ro(dev, 0);
	}
	
	bc->bc_flags	|= query.flags & BC_USER_FLAGS_MASK;
	bc->bc_flags	|= BC_FLAGS_DEV_BUSY;
	bc->bc_offset	= query.offset;
	bc->bc_key	= query.key_handle;
	bc->bc_self_dev	= dev;
	bc->bc_alg	= bc_alg;
	bc->bc_dentry	= dget(file->f_dentry);
//	bc->bc_dentry->d_inode->i_count++;
	bc->bc_file	= file; 

	figure_bc_size(bc);
	
	MOD_INC_USE_COUNT;

	return 0;
}

static int bc_clr_fd(struct bc_device *bc, kdev_t dev)
{
	if (!bc->bc_dentry)
		return -ENXIO;
	if (bc->bc_refcnt > 1)	/* we needed one fd for the ioctl */
		return -EBUSY;
	if (S_ISBLK(bc->bc_dentry->d_inode->i_mode))	
		blkdev_release (bc->bc_dentry->d_inode);
	dput(bc->bc_dentry);	/* release inode */
	fput(bc->bc_file);
	
	bc->bc_flags    = 0;
	bc->bc_dev      = 0;
	bc->bc_self_dev = 0;
	bc->bc_dentry   = NULL;
	bc->bc_file     = NULL;
	bc->bc_offset   = 0;
	bc_sizes[bc->bc_number] = 0;
	invalidate_buffers(dev);
	bc->bc_alg->lock_key(bc->bc_key, 0);
	bc->bc_alg->free_key(bc->bc_key);
	bc->bc_alg = NULL;
	bc->bc_key = 0;
	MOD_DEC_USE_COUNT;

	return 0;
}

static int bc_lock_dev(struct bc_device * bc, int lock)
{
	struct super_block * sb;

	if (!(bc->bc_flags & BC_FLAGS_DEV_BUSY)) {
		if (!lock) 
			return 0;
		printk("bc: attempt to lock free device.\n");
		return -EINVAL;
	}
	sb = get_super(bc->bc_self_dev);
	if (!sb || !sb->s_root) {
		if (!lock)
			return 0;
		printk("bc: attempt to lock unmounted device.\n");
		return -EINVAL;
	}

	if (lock && !(bc->bc_flags & BC_FLAGS_DEV_LOCK)) {
	    sb->s_root->d_count++;
	    bc->bc_flags |= BC_FLAGS_DEV_LOCK;
	} else if (!lock && bc->bc_flags & BC_FLAGS_DEV_LOCK) {
	    sb->s_root->d_count--;
	    bc->bc_flags &= ~BC_FLAGS_DEV_LOCK;
	}
	return 0;
}

/*-- algorithm driver ioctls --------------------------------*/

static int bc_verify_alg(struct bc_alg *arg)
{
	struct bc_alg		query;
	struct bc_algorithm	*bc_alg;

	if (!arg)
		return -EINVAL;
	if (copy_from_user(&query, arg, sizeof(query)))
		return -EFAULT;

	if (query.magic != BC_MAGIC) 
		return -EINVAL;

	bc_alg = get_bc_algo(query.alg_id, &query.alg_module[0]);
	if (!bc_alg)
		return -EINVAL;

	return 0;	
}

static int bc_make_key(struct bc_key *arg)
{
	struct bc_key		query;
	struct bc_algorithm	*bc_alg;
	int			error;

	if (!arg)	
		return -EINVAL;
	if (copy_from_user(&query, arg, sizeof(query)))
		return -EFAULT;

	if (query.magic != BC_MAGIC) 
		return -EINVAL;
	if (!query.key || !query.pool) 
		return -EINVAL;	

	bc_alg = get_bc_algo(query.alg_id, &query.alg_module[0]);
	if (!bc_alg) 
		return -EINVAL;

	/* WARNING!!! using user mode memory areas */
	error = bc_alg->make_key(query.key, query.key_len, query.pool, &query.key_handle);
	if (error)
		return error;	

	if (copy_to_user(&arg->key_handle, &query.key_handle, sizeof(KEY_HANDLE)))
		return -EFAULT;

	return 0;	
}

static int bc_free_key(struct bc_key *arg)
{
	struct bc_key		query;
	struct bc_algorithm	*bc_alg;
	int			error;

	if (!arg)	
		return -EINVAL;
	if (copy_from_user(&query, arg, sizeof(query)))
		return -EFAULT;

	if (query.magic != BC_MAGIC) 
		return -EINVAL;

	bc_alg = get_bc_algo(query.alg_id, NULL);
	if (!bc_alg) 
		return -EINVAL;
	error = bc_alg->free_key(query.key_handle);
	return error;
}

static int bc_process_block(struct bc_block *arg, int operation)
{
	struct bc_block		query;
	struct bc_algorithm	*bc_alg;
	int			error;
	char			*buffer;

	if (!arg)	
		return -EINVAL;
	if (copy_from_user(&query, arg, sizeof(query)))
		return -EFAULT;

	if (query.magic != BC_MAGIC) 
		return -EINVAL;
	if (!query.buffer) 
		return -EINVAL;
	if (query.buffer_len > MAX_BLOCK_SIZE) 
		return -EINVAL;
	
	bc_alg = get_bc_algo(query.alg_id, NULL);
	if (!bc_alg) 
		return -EINVAL;

/*	buffer = kmalloc(query.buffer_len, GFP_KERNEL);*/
	buffer = vmalloc(query.buffer_len);
	if (!buffer) 
		return -ENOMEM;
	
	if (copy_from_user(buffer, query.buffer, query.buffer_len)) {
		error = -EFAULT;
		goto error_out;
	}

	switch (operation) {
	case BC_ENCRYPT_BLOCK:
		bc_alg->encrypt(query.key_handle, &query.iv[0], 
				buffer, buffer, query.buffer_len);
		break;
	case BC_DECRYPT_BLOCK:
		bc_alg->decrypt(query.key_handle, &query.iv[0], 
				buffer, buffer, query.buffer_len);
		break;
	default:
		error = -EINVAL;
		goto error_out;
	}
	
	error = copy_to_user(query.buffer, buffer, query.buffer_len);

error_out:
	if (buffer) {
		memset(buffer, 0, query.buffer_len);
/*		kfree(buffer); */
		vfree(buffer);
		buffer = NULL;
	}
	return error;
}





static struct proc_dir_entry	proc_bc = {
	low_ino:  0,			/* dynamic inode */
	namelen:  6,			/* name length   */
	name:     "bcrypt",
	mode:     S_IFREG | S_IRUGO,
	nlink:    1,			/* # of links    */
	uid:      0, 
	gid:      0,
	size:     0,
	ops:      NULL,			/* i_ops, leaf   */
	get_info: proc_bc_info		/* get_info      */
};

EXPORT_SYMBOL(register_bc_algo);
EXPORT_SYMBOL(unregister_bc_algo);
MODULE_AUTHOR("Jetico, Inc.");

/*-----------------------------------------------------------*/
int init_module(void) 
{
	int	rc;
	struct file_operations *fops;
	struct inode		fake_inode;

	printk("Loading %s ...\n", BC_VERSION_STRING);
	memset(&fake_inode, 0, sizeof(struct inode));
	fake_inode.i_rdev = MKDEV(MAJOR_NR, BC_FIRST);
	fops = get_blkfops(MAJOR_NR);
	if (NULL == fops) {
		if (0 > register_blkdev(MAJOR_NR, DEVICE_NAME, &bc_fops)) {
			printk("bc: unable to get major number %d\n", MAJOR_NR);
			return -EIO;
		}
		
	} else {
		if (-ENODEV != fops->open(&fake_inode, NULL)) {
			fops->release(&fake_inode, NULL);
			printk("bc: already loaded\n");
			return -EIO;
		}
		for (max_loop = 255; max_loop >= 0; max_loop--) {
			fake_inode.i_rdev = MKDEV(MAJOR_NR, max_loop);
			if (-ENODEV != fops->open(&fake_inode, NULL))
				break;
		}
		if ((max_loop < 0) || (max_loop >= BC_FIRST)) {
			printk("bc: initialization error\n");
			return -EIO;
		}
		max_loop++;
		
		memcpy(&save_fops, fops, sizeof(struct file_operations));
		memcpy(fops, &bc_fops, sizeof(struct file_operations));
		save_request_fn = blk_dev[MAJOR_NR].request_fn;
		save_blksize = blk_size[MAJOR_NR];
		save_blksize_size = blksize_size[MAJOR_NR];
	}

	for (rc = 0; rc < MAX_BC_DEVS; rc++) {
		memset(&bc_dev[rc], 0, sizeof(struct bc_device));
		bc_dev[rc].bc_number = rc+BC_FIRST;
	}
	memset(&bc_sizes, 0, sizeof(bc_sizes));
	memset(&bc_blksizes, 0, sizeof(bc_blksizes));

	if (0 > init_bc_algo()) {
		rc = -EINVAL; 
		goto error_out;
	}
	proc_register(&proc_root, &proc_bc);


	if (max_loop) {
		if (blk_size[MAJOR_NR])
			memcpy(bc_sizes, blk_size[MAJOR_NR], (max_loop)*sizeof(int));
		if (blksize_size[MAJOR_NR])
			memcpy(bc_blksizes, blksize_size[MAJOR_NR], (max_loop)*sizeof(int));
	}
	blk_dev[MAJOR_NR].request_fn = do_bc_request;
	blk_size[MAJOR_NR] = bc_sizes;
	blksize_size[MAJOR_NR] = bc_blksizes;
	return 0;
	
error_out:
	if (NULL == fops) {
		unregister_blkdev(MAJOR_NR, DEVICE_NAME);	
	} else {
		memcpy(fops, &save_fops, sizeof(struct file_operations));
	}
	return rc;
}

void cleanup_module(void) 
{
	struct file_operations *fops;
	struct inode fake_inode;

	if (max_loop) {
		fops = get_blkfops(MAJOR_NR);
		memcpy(fops, &save_fops, sizeof(struct file_operations));
		blk_dev[MAJOR_NR].request_fn = save_request_fn;
		memcpy(save_blksize, blk_size[MAJOR_NR], max_loop*sizeof(int));
		memcpy(save_blksize_size, blksize_size[MAJOR_NR], max_loop*sizeof(int));
		blk_size[MAJOR_NR] = save_blksize;
		blksize_size[MAJOR_NR] = save_blksize_size;
	
		memset(&fake_inode, 0, sizeof(struct inode));
		fake_inode.i_rdev = MKDEV(MAJOR_NR, max_loop-1);
		fops->release(&fake_inode, NULL);	
	} else {
		if (unregister_blkdev(MAJOR_NR, DEVICE_NAME) != 0) 
		printk("bc: cleanup failed.\n");
	}
	proc_unregister(&proc_root, proc_bc.low_ino);
}
/*-- end of device registration stuff -----------------------*/
