/*******************************************************************
 *	Copyright (c) 1994-1998 Jetico, Inc., Finland
 *	All rights reserved.
 *
 *	File:		driver/bc.h
 *
 *	Description:	device structure
 *
 *	Scope:		BestCrypt pseudo-device driver
 *
 *	Platforms:	Linux
 * 
 *	Author:		Nail R. Kaipov
 *
 *	Created:	10-Nov-1998
 *
 *	Revision:	
 *
 *******************************************************************/

#ifndef __BC__
#define __BC__

#ifdef __KERNEL__

/* bc device description */
struct bc_device {
	int	bc_number;			/* bc device number */
	int	bc_refcnt;			/* references count */
	int	bc_flags;			/* bc flags         */
	int	bc_offset;			/* data offset      */

	struct file		*bc_file;	/* */
	struct dentry		*bc_dentry;	/* bc special dentry */
	kdev_t	bc_dev;				/* real device      */
	kdev_t	bc_self_dev;			/* self device      */
	KEY_HANDLE		bc_key;		/* key handle       */
	struct bc_algorithm	*bc_alg;	/* encryption algo  */
};


#define DEVICE_NAME	"loop"			/* masquerade	    */
#define MAJOR_NR	7			/* loop device      */
#define MAX_BC_DEVS	8			/* # of bc devices  */
#define MAX_MINORS	256			/* # of minors      */
#define BC_FIRST	128			/* first bc minor   */
#define BC_LAST		(BC_FIRST+(MAX_BC_DEVS-1))
#define MAX_BC_REQUESTS	2			/* private queue    */

extern struct file_operations *get_blkfops(unsigned int);

#define BC_SYSTEM_FLAGS_MASK	0xFFFF0000
#define BC_USER_FLAGS_MASK	0x0000FFFF
/* System flags definition. See bc_ioctl.h for user flags */
#define BC_FLAGS_DO_BMAP	0x00010000	/* disk image is a file */
#define BC_FLAGS_LOOP_DEV	0x00020000	/* loop driver compatibility mode*/
#define BC_FLAGS_DEV_BUSY	0x10000000	/* device busy */
#define BC_FLAGS_DEV_LOCK	0x20000000	/* mounted device is locked */
/* User flags are moved to bc_ioctl.h */


#endif /* __KERNEL__ */

#endif /* __BC__ */
