/*******************************************************************
*
*      Copyright (c) 1994-1999 Jetico, Inc., Finland
*      All rights reserved.
*
*      File: bccreate.c
*      Revision:
*      Created:
*      Description: implementation of create container routine
*
*******************************************************************/
	    
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <strings.h>
#include <string.h>
#include <bc_types.h>
#include <bc_ioctl.h>
#include <kgsha.h>
#include <kg_defs.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "header.h"
#include <kblock.h>
#include "alg.h"
#include "config.h"
#include "bctool.h"
#include "misc.h"
#include "error.h"


ALG_SERV Alg;
DWORD KeyHandle;
char *ContName=NULL;
DATA_BLOCK *pdb=NULL;


void CreateSignalHandler(int sig){
 
   if(ask_y_n("Do you really want to terminate process? (y/n) [n]:")==FALSE)return;
   FreeKeyHandle(Alg,KeyHandle);
   if (pdb!=NULL)FreeDataBlock((BYTE **)&pdb);
   if (ContName!=NULL)unlink(ContName);
   exit(0);
}
 

void bccreate (int argc, char ** argv)
{

   struct HiddenSector 	hs;
   int fd;
   DWORD DataSize;
   alg_info *alg_inf;
   long int i;


   if (argc < 2){ 
      fprintf(stderr,"Please specify container name\n"); 
      exit(0); 
   } else if (argc > 2 ){
      fprintf(stderr,"Too many parameters\n");
      exit(0);
   }
   if (alg==NULL){
      fprintf(stderr,"Please specify algorithm name\n");
      exit(0);
   }

   if (cont_size==0){
      fprintf(stderr,"Please specify container size\n");
      exit(0);
   }

     if ((alg_inf=GetInfoByName(alg))==NULL){
      fprintf(stderr,"Unknown algorithm: %s\n",alg);
      exit(0);
   }

   SetSignalHandlers(CreateSignalHandler);
   
   /* setting defaults for hidden sector */    
   memset(&hs,0,sizeof(hs));
   strncpy(hs.br.OEMid,"LOCOS94 ",8);
   strncpy(hs.br.volumeLabel,"CRYPTED_DSK",11);

   hs.br.jmpCode[0] = 0xEB;
   hs.br.jmpCode[1] = 0x3C;
   hs.br.jmpCode[2] = 0x90;
   
   hs.dwKeySize = sizeof(DATA_BLOCK);
   
   hs.dwDataOffset = ( 512 + sizeof(DATA_BLOCK) + 511 ) & ~511;
   
   hs.br.bpb.totalSectorsLong = (DWORD)((cont_size - hs.dwDataOffset) / 512) ;
   hs.description[0]='\0';
   if (description!=NULL){
      strncpy(hs.description,description,65);
      hs.description[65]=0;
   }
   hs.algorithmId=alg_inf->id;
   hs.keyGenId=KGSHA_ID;
   /* end of setting defaults */   
   
   if ((Alg.bc_fd = open("/dev/bcrypt0", O_RDONLY)) < 0) {
      perror("/dev/bcrypt0");
      exit(0);
    }
   if ((fd = open(argv[1], O_RDWR|O_CREAT|O_EXCL)) < 0){ 
      perror (argv[1]);
      exit (0); 
   }
   ContName=argv[1];
   
   Alg.alg_id=alg_inf->id;
   Alg.alg_module =alg_inf->mod_name;
   DataSize = sizeof(DATA_BLOCK);
   
   CreateKeyHandle(Alg,alg_inf->key_size,NULL,"Enter password:",CFLAG_CREATE_WITH_SINGLE_PASSWORD,(BYTE **)&pdb,&DataSize,&KeyHandle,&Error);
   if (Error!=0){
      print_error(Error);
      exit(0);
   }
   if (write(fd,&hs,sizeof(hs)) < 0){ 
      perror (argv[1]);
      FreeKeyHandle(Alg,KeyHandle);
      FreeDataBlock((BYTE **)&pdb);
      close(fd);
      unlink(argv[1]);
      exit (0); 
   }
   if (write(fd,pdb,sizeof(DATA_BLOCK)) < 0){ 
      perror (argv[1]);
      FreeKeyHandle(Alg,KeyHandle);
      FreeDataBlock((BYTE **)&pdb);
      close(fd);
      unlink(argv[1]);
      exit (0); 
   }
   memset (pdb,0,sizeof(DATA_BLOCK));
   cont_size-=sizeof(hs)+sizeof(DATA_BLOCK);
   for (i=cont_size;i>=sizeof(DATA_BLOCK);i-=sizeof(DATA_BLOCK)){
      if (write(fd,pdb,sizeof(DATA_BLOCK)) < 0){ 
	 perror (argv[1]);
	 FreeKeyHandle(Alg,KeyHandle);
	 FreeDataBlock((BYTE **)&pdb);
	 close(fd);
	 unlink(argv[1]);
	 exit (0); 
      }
   }
   if (write(fd,pdb,i) < 0){ 
      perror (argv[1]);
      FreeKeyHandle(Alg,KeyHandle);
      FreeDataBlock((BYTE **)&pdb);
      close(fd);
      unlink(argv[1]);
      exit (0); 
   }

   FreeKeyHandle(Alg,KeyHandle);
   FreeDataBlock((BYTE **)&pdb);
   
   fchmod(fd,S_IRUSR|S_IWUSR);
   
   close(fd);
   close(Alg.bc_fd);

}
