/*******************************************************************
*
*      Copyright (c) 1994-1999 Jetico, Inc., Finland
*      All rights reserved.
*
*      File: bcformat.c
*      Revision:
*      Created:
*      Description: implementation of format container routine
*
*******************************************************************/
 

#include <sys/mount.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <bc_types.h>
#include <bc_ioctl.h>
#include <kgsha.h>
#include <kg_defs.h>
#include <fcntl.h>
#include "header.h"
#include <kblock.h>
#include "alg.h"
#include <string.h>
#include "config.h"
#include "bctool.h"
#include "misc.h"
#include "error.h"

void bcformat (int argc, char ** argv)
{

   struct HiddenSector 	hs;
   char *device;
   int fd,err;
   long Offset;
   ALG_SERV Alg;
   DWORD KeyHandle,DataSize;
   DATA_BLOCK db, *pdb;
   alg_info *alg_inf;
   char *command[5];

   if (argc < 2){ 
      fprintf(stderr,"Please specify container name\n"); 
      exit(0); 
   } else if (argc > 2 ){
      fprintf(stderr,"Too many parameters\n");
      exit(0);
   }

   if (fstype==NULL){
      printf("Filesystem type not specified. Msdos used.\n");
      fstype="msdos";
   }


   if((device=FindFreeDevice())==NULL){
      fprintf(stderr,"Can not find free device\n");
      exit(0);
   }
    
   if ((Alg.bc_fd = open(device, O_RDONLY)) < 0) {
      perror(device);
      exit(0);
    }
   if ((fd = open(argv[1], O_RDWR)) < 0){ 
      perror (argv[1]);
      exit (0); 
   }
   if (read(fd, &hs, sizeof(hs))!=sizeof(hs)){
      fprintf(stderr,"Can not read container header\n");
      exit (0); 
   }
   if (strncmp(hs.br.OEMid, "LOCOS94 ", sizeof(hs.br.OEMid))!=0){
      fprintf(stderr,"%s is not valid BestCrypt container\n",argv[0]);
      exit (0);
    };   
   if (read(fd, &db, sizeof (db))!=sizeof(db)){
      fprintf(stderr,"Can not read data block\n");
      exit (0); 
   }
   if ((alg_inf=GetInfoById(hs.algorithmId))==NULL){
      fprintf(stderr,"Unknown algorithm Id: %d\n",hs.algorithmId);
      exit(0);
   }
   
   Alg.alg_id=hs.algorithmId;
   Alg.alg_module =alg_inf->mod_name;
   pdb = &db;
   DataSize = sizeof(db);
   Offset=hs.dwDataOffset;
   
   CreateKeyHandle(Alg,alg_inf->key_size,NULL,"Enter password:",CFLAG_VERIFY_AND_LOAD_KEY,(BYTE **)&pdb,&DataSize,&KeyHandle,&Error);
   if (Error!=0){
      print_error(Error);
      exit(0);
   }
   
   Error=bc_set_fd_kh(Alg, KeyHandle, fd, Offset, 0);
   if (Error!=0){
      print_error(Error);
      FreeKeyHandle(Alg,KeyHandle);
      exit(0);
   }

   command[0]="/sbin/mkfs";
   command[1]="-t";
   command[2]=fstype;
   command[3]=device;
   command[4]=0;
   err=run_process("/sbin/mkfs",command);
   if (err==-1){
      perror("/sbin/mkfs");
      FreeKeyHandle(Alg,KeyHandle);
      exit(0);
   } else if (err==127) {
      fprintf(stderr,"Can not launch /sbin/mkfs\n");
      FreeKeyHandle(Alg,KeyHandle);
      exit(0);
   }
   
   FreeKeyHandle(Alg,KeyHandle);
   
   hs.fileSystemId=getIdByFStype(fstype);
   
   Error=bc_clear_fd(Alg);
   if (Error!=0){
      print_error(Error);
      exit(0);
   }

   if (lseek(fd,0,SEEK_SET) < 0){
      perror (argv[1]);
      exit (0);
   }
   if (write(fd,&hs,sizeof(hs)) < 0){
      perror (argv[1]);
      exit (0);
   }
   
   close(fd);
   close(Alg.bc_fd);
}

