/*******************************************************************
*
*      Copyright (c) 1994-1999 Jetico, Inc., Finland
*      All rights reserved.
*
*      File: bccreate.c
*      Revision:
*      Created:
*      Description: implementation of mount container routine
*
*******************************************************************/
 


#include <sys/stat.h>
#include <stdlib.h>
#include <dirent.h>
#include "header.h"
#include <errno.h>
#include <stdio.h>
#include <mntent.h>
#include <unistd.h>
#include <sys/mount.h>
#ifndef MS_MGC_VAL
  #include <linux/fs.h>
#endif
#include <bc_types.h>
#include <bc_ioctl.h>
#include <kgsha.h>
#include <kg_defs.h>
#include <fcntl.h>
#include <kblock.h>
#include "alg.h"
#include <string.h>
#include "config.h"
#include "bctool.h"
#include "misc.h"
#include "error.h"

void bcmount (int argc, char ** argv)
{

   struct HiddenSector 	hs;
   char *device;
   int fd;
   long Offset;
   ALG_SERV Alg;
   DWORD KeyHandle,DataSize;
   DATA_BLOCK db, *pdb;
   alg_info *alg_inf;

   FILE *mtab;
   struct mntent mnt;
   char rp1[MAXNAMLEN];
   char rp2[MAXNAMLEN];
   char options[256];
   unsigned long flags;
		   


   if (argc < 3){ 
      fprintf(stderr,"Please specify container name and mount point\n"); 
      usage(stdin);
      exit(0);
   } else if (argc > 3 ){
      fprintf(stderr,"Too many parameters\n");
      usage(stdin);
      exit(0);
   }

	Error=IsOwner(ruid,argv[2]);
    if (Error==FALSE && ruid!=0){
		msg(stderr,"sorry, you are not owner of this mount point");
		exit(0);
    } else if (Error==-1){
		print_error(errno);
		exit(0);
    }

   if((device=FindFreeDevice())==NULL){
      fprintf(stderr,"Can not find free device\n");
      exit(0);
   }
    
   if ((Alg.bc_fd = open(device, O_RDONLY)) < 0) {
      perror(device);
      exit(0);
    }
   if ((fd = open(argv[1], O_RDONLY,O_SYNC)) < 0){ 
      perror (argv[1]);
      exit (0); 
   }
   if (read(fd, &hs, sizeof(hs))!=sizeof(hs)){
      fprintf(stderr,"Can not read container header\n");
      exit (0); 
   }
   if (strncmp(hs.br.OEMid, "LOCOS94 ", sizeof(hs.br.OEMid))!=0){
      fprintf(stderr,"%s is not valid BestCrypt container\n",argv[0]);
      exit (0);
    };   
   if (read(fd, &db, sizeof (db))!=sizeof(db)){
      fprintf(stderr,"Can not read data block\n");
      exit (0); 
   }
   if ((alg_inf=GetInfoById(hs.algorithmId))==NULL){
      fprintf(stderr,"Unknown algorithm Id: %d\n",hs.algorithmId);
      exit(0);
   }
   
   Alg.alg_id=hs.algorithmId;
   Alg.alg_module =alg_inf->mod_name;
   pdb = &db;
   DataSize = sizeof(db);
   Offset=hs.dwDataOffset;
   
   CreateKeyHandle(Alg,alg_inf->key_size,NULL,"Enter password:",CFLAG_VERIFY_AND_LOAD_KEY,(BYTE **)&pdb,&DataSize,&KeyHandle,&Error);
   if (Error!=0){
      print_error(Error);
      exit(0);
   }
   
   Error=bc_set_fd_kh(Alg, KeyHandle, fd, Offset, 0);
   if (Error!=0){
      print_error(Error);
      FreeKeyHandle(Alg,KeyHandle);
      exit(0);
   }
   if (fstype==NULL && (fstype=getFStypeById(hs.fileSystemId))==NULL){
      fprintf(stderr,"Unknown filesystem. Try: -t fstype \n");
      bc_clear_fd(Alg);
      FreeKeyHandle(Alg,KeyHandle);
      exit(0);
   }

   if (setreuid(euid,euid)!=0){ 
      msg(stderr,"setreuid before mount:");
      print_error(errno); 
      bc_clear_fd(Alg);
      FreeKeyHandle(Alg,KeyHandle);
      exit(0);
   }

	sprintf(options,"uid=%d,umask=077",ruid);
	flags=MS_MGC_VAL|(MS_RDONLY & ro);
	
	//
	// disable suid and dev files if not root 
	//
	if (ruid!=0)
		flags|=MS_NOSUID|MS_NODEV;
    	
   if (mount(device,argv[2],fstype,flags,options)!=0 &&
 		mount(device,argv[2],fstype,flags,NULL)!=0)
   {
      print_error(errno);
      bc_clear_fd(Alg);
      FreeKeyHandle(Alg,KeyHandle);
      exit (0);
   }

	
   Error=bc_lock_dev(Alg);
   if (Error!=0){
      print_error(Error);
      exit(0);
   }


   if((mnt.mnt_fsname=realpath(argv[1],rp1))==NULL) print_error(errno);
   if((mnt.mnt_dir=realpath(argv[2],rp2))==NULL) print_error(errno);
   mnt.mnt_type=fstype;
   mnt.mnt_opts= ro ? "ro,BestCrypt" : "rw,BestCrypt";
   mnt.mnt_freq=0;
   mnt.mnt_passno=0;
   mtab = setmntent("/etc/mtab",  "r+");
   if (mtab != NULL){
      addmntent(mtab, &mnt);
      endmntent(mtab);
   } else msg(stderr,"Can not open /etc/mtab. Mount entry will not be added in mtab file.\n");


   FreeKeyHandle(Alg,KeyHandle);

    close(fd);
    close(Alg.bc_fd);

	chown(rp2,ruid,-1);
	chmod(rp2,0700);
	
}

void bcumount (int argc, char ** argv){

   ALG_SERV Alg;
   char *device;
   FILE *mtab,*tmptab;
   struct mntent *pmnt;
   char rp[MAXNAMLEN];

   if (argc < 2){ 
      fprintf(stderr,"Please specify mount point\n"); 
      usage(stdin);
      exit(0);
   } else if (argc > 2 ){
      fprintf(stderr,"Too many parameters\n");
      usage(stdin);
      exit(0);
   }
   
   if (NULL == realpath(argv[1],rp)){print_error(errno);exit(0);};

 	Error=IsOwner(ruid,rp);
    if (Error==FALSE && ruid!=0){
		msg(stderr,"sorry, you are not owner of this mount point");
		exit(0);
    } else if (Error==-1){
		print_error(errno);
		exit(0);
    }


   device=FindMountedDevice(rp);
   if(device==NULL){
      fprintf(stderr,"Can not find mounted device\n");
      exit(0);
   }

   if ((Alg.bc_fd = open(device, O_RDONLY)) < 0) {
      perror(device);
      exit(0);
    }

   Error=bc_unlock_dev(Alg);
   if (Error!=0){
      print_error(Error);
      exit(0);
   }

   if (setreuid(euid,euid)!=0){ 
      msg(stderr,"setreuid before umount:");
      print_error(errno); 
      exit(0);
   }

   if (umount(device)!=0){
      perror("umount");
      exit (0);
   }

    mtab   = setmntent("/etc/mtab",  "r");
    tmptab = setmntent("/etc/mtab.tmp", "w+");
 
    while(1){
        pmnt=getmntent(mtab);
        if (feof(mtab))break;
        if (strcmp(rp,pmnt->mnt_dir))
        addmntent(tmptab, pmnt);
    }
 
    endmntent(mtab);
    endmntent(tmptab);
 
    if (rename("/etc/mtab","/etc/mtab~")!=0) perror("rename");
    if (rename("/etc/mtab.tmp","/etc/mtab")!=0) perror("rename");
    if (chmod("/etc/mtab",0644)!=0) perror("chmod");
    if (unlink("/etc/mtab~")!=0) perror("unlink");

   Error=bc_clear_fd(Alg);
   if (Error!=0){
      print_error(Error);
      exit(0);
   }



}



