/*******************************************************************
*
*      Copyright (c) 1994-1999 Jetico, Inc., Finland
*      All rights reserved.
*
*      File: bccreate.c
*      Revision:
*      Created:
*      Description: implementation of change password routine
*
*******************************************************************/
 

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <strings.h>
#include <string.h>
#include <bc_types.h>
#include <bc_ioctl.h>
#include <kgsha.h>
#include <kg_defs.h>
#include "header.h"
#include <kblock.h>
#include "alg.h"
#include "config.h"
#include "bctool.h"
#include "misc.h"
#include "error.h"

void bcpasswd (int argc, char ** argv)
{

   struct HiddenSector 	hs;
   int fd;
   ALG_SERV Alg;
   DWORD KeyHandle,DataSize;
   DATA_BLOCK db, *pdb;
   alg_info *alg_inf;


   if (argc < 2){ 
      fprintf(stderr,"Please specify container name\n"); 
      exit(0); 
   } else if (argc > 2 ){
      fprintf(stderr,"Too many parameters\n");
      exit(0);
   }
    
   if ((Alg.bc_fd = open("/dev/bcrypt0", O_RDONLY)) < 0) {
      perror("/dev/bcrypt0");
      exit(0);
    }
   if ((fd = open(argv[1], O_RDWR)) < 0){ 
      perror (argv[1]);
      exit (0); 
   }
   if (read(fd, &hs, sizeof(hs))!=sizeof(hs)){
      fprintf(stderr,"Can not read container header\n");
      exit (0); 
   }
   if (strncmp(hs.br.OEMid, "LOCOS94 ", sizeof(hs.br.OEMid))!=0){
      fprintf(stderr,"%s is not valid BestCrypt container\n",argv[0]);
      exit (0);
    };   
   if (read(fd, &db, sizeof (db))!=sizeof(db)){
      fprintf(stderr,"Can not read data block\n");
      exit (0); 
   }
   if ((alg_inf=GetInfoById(hs.algorithmId))==NULL){
      fprintf(stderr,"Unknown algorithm Id: %d\n",hs.algorithmId);
      exit(0);
   }
   
   Alg.alg_id=hs.algorithmId;
   Alg.alg_module =alg_inf->mod_name;
   pdb = &db;
   DataSize = sizeof(db);
   
   CreateKeyHandle(Alg,alg_inf->key_size,NULL,"Enter password:",CFLAG_CHANGE_PASSWORD,(BYTE **)&pdb,&DataSize,&KeyHandle,&Error);
   if (Error!=0){
      print_error(Error);
      exit(0);
   }
   if (lseek(fd,sizeof(hs),SEEK_SET) < 0){ 
      perror (argv[1]);
      FreeKeyHandle(Alg,KeyHandle);
      exit (0); 
   }
   if (write(fd,&db,sizeof(db)) < 0){ 
      perror (argv[1]);
      FreeKeyHandle(Alg,KeyHandle);
      exit (0); 
   }
   
   
   FreeKeyHandle(Alg,KeyHandle);

   close(fd);
   close(Alg.bc_fd);
}

