/*******************************************************************
*
*      Copyright (c) 1994-1999 Jetico, Inc., Finland
*      All rights reserved.
*
*      File: bccreate.c
*      Revision:
*      Created:
*      Description: implementation of container reencryption routine
*
*******************************************************************/


#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <strings.h>
#include <string.h>
#include <stdlib.h>
#include <bc_types.h>
#include <bc_ioctl.h>
#include <kgsha.h>
#include <kg_defs.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include "header.h"
#include <kblock.h>
#include "alg.h"
#include "config.h"
#include "bctool.h"
#include "misc.h"
#include "error.h"

#define BUFF_SIZE 1024*1024 // 1 Mb





char *fnbuff,*fnstat1,*fnstat2,*fnheader1,*fnheader2;
ALG_SERV AlgOld,AlgNew;
DWORD KeyHandleOld,KeyHandleNew;



struct _state{
   unsigned int step; //1:buffer empty(redy to fill bufer)
                      //2:buffer empty(redy to write data in container)
                      //3: reencryption done(ready to write header)
                      //4: all done
   int dir;           // 1: begin to end; -1: end to begin; 0: read done
   unsigned int  BuffSize;
   unsigned int  OldDataOffset; // also = old header size
   unsigned int  NewDataOffset; // also = new header size
   long long int ReadPos;
   long long int WritePos;
   long long int OldContSize;
   long long int NewContSize;
   
}__attribute__ ((packed)) state;



void ReencryptSignalHandler(int sig){

   if(ask_y_n("Do you really want to terminate process? (y/n) [n]:")==FALSE)return;
   FreeKeyHandle(AlgNew,KeyHandleNew);
   FreeKeyHandle(AlgOld,KeyHandleOld);
   exit(0);
}





int WriteStatFile(char *fileName, void *buffer, unsigned int size)
{
   int fd;

   fd = open(fileName,O_WRONLY|O_CREAT|O_SYNC,S_IRUSR|S_IWUSR);
   if (fd==-1) return FALSE;
   
   if ( write(fd,buffer,size) != size) return FALSE;
   
   if ( close( fd )==-1) return FALSE;

   return TRUE;
};

int ReadStatFile(char *fileName, void *buffer, DWORD size )
{
   
   struct stat buf;
   int fd;
   
   if (lstat(fileName,&buf)==-1) return FALSE;
   
   if ( buf.st_size != size ) return FALSE;
   
   fd = open(fileName,O_RDONLY);
   if (fd == -1) return FALSE;
   
   if (read(fd,buffer,size)!=size) {
      close(fd);
      return FALSE;
   }
   
   close(fd);
   return TRUE;
}


int PrepareFileNames(char *ContName){
 
   fnbuff=(char*)malloc(strlen(ContName)+7);
   fnstat1=(char*)malloc(strlen(ContName)+7);
   fnstat2=(char*)malloc(strlen(ContName)+7);
   fnheader1=(char*)malloc(strlen(ContName)+7);
   fnheader2=(char*)malloc(strlen(ContName)+7);
   if (fnbuff==NULL||fnstat1==NULL||fnstat2==NULL||fnheader1==NULL||fnheader2==NULL) return FALSE;
 
   sprintf(fnbuff,"%s.bfr",ContName);
   sprintf(fnstat1,"%s.st1",ContName);
   sprintf(fnstat2,"%s.st2",ContName);
   sprintf(fnheader1,"%s.hr1",ContName);
   sprintf(fnheader2,"%s.hr2",ContName);
 
   return TRUE;
 
}

void FreeFileNames(){
   free(fnbuff);
   free(fnstat1);
   free(fnstat2);
   free(fnheader1);
   free(fnheader2);
}


int UnlinkStatFiles(){
   
   if (unlink(fnbuff)==0&&unlink(fnstat1)==0&&unlink(fnstat2)==0&&unlink(fnheader1)==0&&unlink(fnheader2)==0)return 0;
   return errno;
}



/*********************************************************
 *  init 
 * 
 * 
**********************************************************/
int init(char *ContName,char *DB,int DB_size,unsigned int KgId, unsigned int AlgId){
   
   int fd;
   struct HiddenSector hs;
   void *buff;

   if (DB==NULL||ContName==NULL)return FALSE;

   if ((fd = open(ContName, O_RDONLY)) < 0) return FALSE;
   if (read(fd, &hs, sizeof(hs))!=sizeof(hs)) return FALSE;
      
   state.OldDataOffset=hs.dwDataOffset;
   state.OldContSize=hs.br.bpb.totalSectorsLong*512L + hs.dwDataOffset;
   state.NewDataOffset=(sizeof(hs)+DB_size +511 ) & ~511;
   state.NewContSize=hs.br.bpb.totalSectorsLong*512L + state.NewDataOffset ;
   state.step=1;
   state.BuffSize=BUFF_SIZE;
   state.dir=state.OldDataOffset>= state.NewDataOffset ? 1 : -1;
   state.ReadPos  = state.dir>0 ? state.OldDataOffset : state.OldContSize-state.BuffSize;
   state.WritePos = state.dir>0 ? state.NewDataOffset : state.NewContSize;
   
   lseek(fd,0,SEEK_SET);
   buff=malloc(hs.dwDataOffset); if (buff==NULL)return FALSE;
   if (read(fd,buff,hs.dwDataOffset)!=hs.dwDataOffset) return FALSE; 
   close(fd);
   if (WriteStatFile(fnheader1,buff,hs.dwDataOffset)==FALSE)return FALSE;
   free(buff);
   
   // mark reencrypted
   strncpy(hs.br.volumeLabel,"REENCRYPTED",11);
   if (WriteStatFile(ContName,&hs,sizeof(hs))==FALSE)return FALSE;

   // preparing new HiddenSector
   hs.dwKeySize = DB_size;
   strncpy(hs.br.volumeLabel,"CRYPTED_DSK",11);
   hs.algorithmId=AlgId;
   hs.keyGenId=KgId;
   hs.dwDataOffset=state.NewDataOffset;
   buff=malloc(state.NewDataOffset);
   if (buff==NULL)return FALSE;
   memset(buff,0,state.NewDataOffset);
   memcpy(buff,&hs,sizeof(hs));
   memcpy(buff+sizeof(hs),DB,DB_size);
   if (WriteStatFile(fnheader2,buff,state.NewDataOffset)==FALSE)return FALSE;
   free(buff);
   
   return WriteStatFile(fnstat1,&state,sizeof(state))&&WriteStatFile(fnstat2,&state,sizeof(state));
   
}


int WasInterrupted(char *ContName){
   
   struct stat s;
   
   return (lstat(fnheader1,&s)!=-1 && lstat(fnheader1,&s)!=-1 && (lstat(fnstat1,&s)!=-1 || lstat(fnstat2,&s)!=-1)) ? TRUE : FALSE;
}

int WriteStatFiles(){
   return (WriteStatFile(fnstat1,&state,sizeof(state)) && WriteStatFile(fnstat2,&state,sizeof(state)));
}

int ReadStatFiles(){
   return ReadStatFile(fnstat1,&state,sizeof(state)) || ReadStatFile(fnstat2,&state,sizeof(state));
}


int WriteContFile(int fd, void *buff){
   int size;
   
   if (state.BuffSize==0)return 0;
   
   if (lseek(fd,state.WritePos,SEEK_SET)==-1)return -1;
   if ((size=write(fd,buff,state.BuffSize))==-1)return -1;
   return size;

}

int ReadContFile(int fd, void *buff){
   int size;
   
   if (lseek(fd,state.ReadPos,SEEK_SET)==-1)return -1;
   if ((size=read(fd,buff,state.BuffSize))==-1)return -1;
 
   return size;
}

int reencrypt(void *buff){
   
   short IV[4];
   int i;
   
   if (state.BuffSize==0)return TRUE;
   
   for (i=0;i<state.BuffSize;i+=512){
      IV[0]=IV[1]=IV[2]=IV[3]=((state.ReadPos-state.OldDataOffset+i)>>9)&0xFFFF;
      if (alg_decrypt(AlgOld,KeyHandleOld,(BYTE *)IV,buff+i,512)!=0) return FALSE;
      if (alg_encrypt(AlgNew,KeyHandleNew,(BYTE *)IV,buff+i,512)!=0) return FALSE;
   }
   return TRUE;

}

#ifdef 0

void print_stat(){
   printf("Step=%d\n",state.step);
   printf("Dir =%d\n",state.dir);
   printf("ReadPos =%d\n",state.ReadPos);
   printf("WritePos=%d\n",state.WritePos);
   printf("BuffSize=%d\n\n",state.BuffSize);

}

#endif 


int MoveData(char *ContName){
   
   int fd;
   void *buff;
   
   if ((buff=malloc(BUFF_SIZE))==NULL){
      msg(stderr,"Can not allocate memory for buffer\n");
      return FALSE;
   }
   if ((fd = open(ContName, O_RDWR|O_SYNC)) < 0){ 
      msg(stderr,"Can not open container for Read/Write\n");
      return FALSE;
   }
   
   if (ReadStatFiles()==FALSE){
      msg(stderr,"Can not read stat files\n");
      return FALSE;
   }
   
   if (state.step==2){
      if (ReadStatFile(fnbuff,buff,state.BuffSize)==FALSE){
	 msg(stderr,"Can not read data from buffer file\n");
	 close(fd);
	 return FALSE;
      }
   }
   
   do {

      switch (state.step){
       case 1: 
	 if (state.dir==-1 && state.ReadPos-state.OldDataOffset <= 0){
	    state.BuffSize=state.ReadPos+state.BuffSize-state.OldDataOffset;
	    state.ReadPos=state.OldDataOffset;
	    state.WritePos=state.NewDataOffset;
	 }
	 if ((state.BuffSize=ReadContFile(fd,buff))==-1){
	    msg(stderr,"Can not read data from container\n");
	    close(fd);
	    free(buff);
	    return FALSE;
	 }
	 if (reencrypt(buff)==FALSE){
	    msg(stderr,"Buffer reencryption error\n");
            close(fd);
	    free(buff);
            return FALSE; 
	 }
	 if (WriteStatFile(fnbuff,buff,state.BuffSize)==FALSE){
	    msg(stderr,"Can not write data to buffer file\n");
            close(fd);
	    free(buff);
            return FALSE; 
	 }
	 if (state.BuffSize < BUFF_SIZE) state.dir=0;

	 if (state.dir ==  1)state.ReadPos+=state.BuffSize;
	 if (state.dir == -1)state.ReadPos-=state.BuffSize;
	 state.step=2;

	 if (WriteStatFiles()==FALSE){
	    msg(stderr,"Can not write stat files\n");
	    close(fd);
	    free(buff);
	    return FALSE;
	 }
	 break;
       
       case 2:

	 if (state.dir == -1)state.WritePos-=state.BuffSize;
 	 if (WriteContFile(fd,buff)==-1){
	    msg(stderr,"Can not write data to container\n");
	    close(fd);
	    free(buff);
	    return FALSE;
	 }
	 if (state.dir==0) state.step=3; else state.step=1;
	 if (state.dir ==  1)state.WritePos+=state.BuffSize;
	 if (WriteStatFiles()==FALSE){
            msg(stderr,"Can not write stat files\n");
            close(fd);
	    free(buff);
            return FALSE;
	 }
	 break;

       case 3:
	 if (ftruncate(fd,state.NewContSize)==-1){
	    msg(stderr,"Can not truncate container file\n");
	 }
	 close(fd);

	 if (ReadStatFile(fnheader2,buff,state.NewDataOffset)==FALSE){
	    msg(stderr,"Can not read header file\n");
	    free(buff);
	    return FALSE;
	 }
	 if (WriteStatFile(ContName,buff,state.NewDataOffset)==FALSE){
            msg(stderr,"Can not write header\n");
	    free(buff);
            return FALSE;
         }
	 state.step=4;
	 break;
	 
       default:
	 msg(stderr,"Unexpected state\n");
	 free(buff);
	 return FALSE;
      }
   } while (state.step!=4);

   free(buff);
   return TRUE;
}





void bcreencrypt (int argc, char ** argv){
   
   struct HiddenSector  hs;
   int fd;
   DWORD DataSize;
   DATA_BLOCK *pdb,db;
   alg_info *AlgInf;
    
    
   if (argc < 2){
      msg(stderr,"Please specify container name\n");
      return;
   } else if (argc > 2 ){
      msg(stderr,"Too many parameters\n");
      return;
   }

   if (PrepareFileNames(argv[1])==FALSE){
      msg(stderr,"Can not prepare  file names\n");
      return;
   }

   SetSignalHandlers(ReencryptSignalHandler);
   
   if (WasInterrupted(argv[1])==TRUE){
      msg(stdout,"Reencryption process was interrupted\nRestoring reencryption...\n");
      //Restoring reencryption

      if ((AlgOld.bc_fd = open("/dev/bcrypt0", O_RDONLY)) < 0) {
	 msg(stderr,"Can not open /dev/bcrypt0\n");
	 return;
      }
      
      //reading old header
      if ((fd = open(fnheader1, O_RDONLY)) < 0){
	 msg(stderr,"Can not open stata file\n");
	 return;
      }
      if (read(fd, &hs, sizeof(hs))!=sizeof(hs)){
	 msg(stderr,"Can not read old header\n");
	 return;
      }
      
      if (read(fd, &db, sizeof (db))!=sizeof(db)){
	 msg(stderr,"Can not read old data block\n");
	 return;
      }
      close(fd);
      
      if ((AlgInf=GetInfoById(hs.algorithmId))==NULL){
	 msg(stderr,"Unknown old algorithm Id\n");
	 return;
      }
      
      AlgOld.alg_id=hs.algorithmId;
      AlgOld.alg_module =AlgInf->mod_name;
      pdb = &db;
      DataSize = sizeof(db);
      
      CreateKeyHandle(AlgOld,AlgInf->key_size,NULL,"Enter old password:",CFLAG_VERIFY_AND_LOAD_KEY,(BYTE **)&pdb,&DataSize,&KeyHandleOld,&Error);
      if (Error!=0){
	 print_error(Error);
	 return;
      }

      // reading new header
      if ((fd = open(fnheader2, O_RDONLY)) < 0){
	 msg(stderr,"Can not open stat file\n");
	 return;
      }
      if (read(fd, &hs, sizeof(hs))!=sizeof(hs)){
	 msg(stderr,"Can not read hew header\n");
	 return;
      }
      
      if (read(fd, &db, sizeof (db))!=sizeof(db)){
	 msg(stderr,"Can not read new data block\n");
	 return;
      }
      close(fd);
      
      if ((AlgInf=GetInfoById(hs.algorithmId))==NULL){
	 msg(stderr,"Unknown algorithm Id in container file\n");
	 return;
      }
      
      AlgNew.alg_id=hs.algorithmId;
      AlgNew.alg_module =AlgInf->mod_name;
      AlgNew.bc_fd=AlgOld.bc_fd;
      pdb = &db;
      DataSize = sizeof(db);
      
      CreateKeyHandle(AlgNew,AlgInf->key_size,NULL,"Enter new password:",CFLAG_VERIFY_AND_LOAD_KEY,(BYTE **)&pdb,&DataSize,&KeyHandleNew,&Error);
      if (Error!=0){
	 print_error(Error);
	 return;
      }

   // end of restoring
   } else { 
      
      if (alg==NULL){
	 msg(stderr,"Please specify algorithm name\n");
	 return;
      }
      
      if (is_valid_container(argv[1])==FALSE){
	 msg(stderr,"Invalid container\n");
	 return;
      }

      if ((AlgOld.bc_fd = open("/dev/bcrypt0", O_RDONLY)) < 0) {
	 msg(stderr,"Can not open /dev/bcrypt0\n");
	 return;
      }
      
      if ((fd = open(argv[1], O_RDONLY)) < 0){
	 msg(stderr,"Can not open container\n");
	 return;
      }
      if (read(fd, &hs, sizeof(hs))!=sizeof(hs)){
	 msg(stderr,"Can not read container header\n");
	 return;
      }
      
      if (read(fd, &db, sizeof (db))!=sizeof(db)){
	 msg(stderr,"Can not read data block\n");
	 return;
      }
      close(fd);
      
      if ((AlgInf=GetInfoById(hs.algorithmId))==NULL){
	 msg(stderr,"Unknown algorithm Id in container file\n");
	 return;
      }
      
      AlgOld.alg_id=hs.algorithmId;
      AlgOld.alg_module =AlgInf->mod_name;
      pdb = &db;
      DataSize = sizeof(db);
      
      CreateKeyHandle(AlgOld,AlgInf->key_size,NULL,"Enter old password:",CFLAG_VERIFY_AND_LOAD_KEY,(BYTE **)&pdb,&DataSize,&KeyHandleOld,&Error);
      if (Error!=0){
	 print_error(Error);
	 return;
      }
      
      if ((AlgInf=GetInfoByName(alg))==NULL){
	 msg(stderr,"Unknown new algorithm\n");
	 return;
      }
      AlgNew.alg_id=AlgInf->id;
      AlgNew.alg_module =AlgInf->mod_name;
      AlgNew.bc_fd=AlgOld.bc_fd;
      
      CreateKeyHandle(AlgNew,AlgInf->key_size,NULL,"Enter new password:",CFLAG_CREATE_WITH_SINGLE_PASSWORD,(BYTE **)&pdb,&DataSize,&KeyHandleNew,&Error);
      if (Error!=0){
	 print_error(Error);
	 return;
      }
      
      if (init(argv[1],(char *)pdb,DataSize,KGSHA_ID,AlgInf->id)==FALSE){
	 msg(stderr,"Can not init reencryption process\n");
	 FreeKeyHandle(AlgOld,KeyHandleOld);
	 FreeKeyHandle(AlgNew,KeyHandleNew);
	 FreeFileNames();
	 return;
	 
      }
   }
   
      
   if (MoveData(argv[1])==FALSE){
      msg(stderr,"Reencryption error\n");
      FreeKeyHandle(AlgNew,KeyHandleNew);
      FreeKeyHandle(AlgOld,KeyHandleOld);
      FreeFileNames();
      return;
   }
      
      FreeKeyHandle(AlgNew,KeyHandleNew);
      FreeKeyHandle(AlgOld,KeyHandleOld);
      if (UnlinkStatFiles()==-1){
	 msg(stderr,"Can non remove temporary files\nPlease remove manualy\n");
	 msg(stderr,strerror(errno));
      }
      FreeFileNames();
      
}


