/*******************************************************************
*
*      Copyright (c) 1994-1999 Jetico, Inc., Finland
*      All rights reserved.
*
*      File: bccreate.c
*      Revision:
*      Created:
*      Description: implementation of command line interface
*
*******************************************************************/


#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <getopt.h>
#include <bc_types.h>
#include "alg.h"
#include "config.h"
#include "error.h"
#include "version.h"
#include "bctool.h"
#include "misc.h"


extern void bccreate(int argc,char **argv);
extern void bcumount(int argc,char **argv);
extern void bcmount(int argc,char **argv);
extern void bcpasswd(int argc,char **argv);
extern void bcinfo(int argc,char **argv);
extern void bcformat(int argc,char **argv);
extern void bcreencrypt(int argc,char **argv);
extern void bcfsck(int argc,char **argv);

int main (int argc, char ** argv)
{
   char c;
   Error=0;
   fstype=NULL;
   alg=NULL;
   options=NULL;
   cont_size=0;
   description=NULL;
   ro=0;
   

   ruid=getuid();
   euid=geteuid();
   if (setreuid(euid,ruid)!=0){ perror("setreuid:"); exit(0); }

   while ((c=getopt(argc,argv,"Vvhrt:a:o:s:d:"))!=EOF){
      switch(c){
       case 'V':
	 printf("BestCrypt version %s\n",VERSION);
	 exit(0);
       case 'v':
	 verbose++;
	 break;
       case 'r':
         ro=0xff;
	 break; 		 
       case 'h':
	 usage(stdout);
	 exit(0);
       case 't':
	 fstype=optarg;
	 break;
       case 'a':
	 alg=strtolower(optarg);
	 break;
       case 'o':
	 options=optarg;
	 break;
       case 'd':
	 description=optarg;
	 break;
       case 's':
	 cont_size=string_to_long(optarg);
	 if (cont_size==0){
	    fprintf(stderr,"Invalid container size\n");
	    exit(0);
	 }
	 break;
       case '?':
       default:
	 usage(stderr);
	 exit(0);
      } /* end of switch */
      
      
      
   } /* end of while */

   argc -= optind;
   argv += optind;

   if (argc==0) {
      fprintf(stderr,"Missed command\n");
      usage(stderr);
      exit(0);
   }

   if ((Error=read_conf())!=0) { print_error(Error); exit(0);};
   
   if (!strcmp(argv[0],"info")) bcinfo(argc,argv);
   else if (!strcmp(argv[0],"new")) bccreate(argc,argv);
   else if (!strcmp(argv[0],"format")) bcformat(argc,argv);
   else if (!strcmp(argv[0],"mount")) bcmount(argc,argv);
   else if (!strcmp(argv[0],"umount")) bcumount(argc,argv);
   else if (!strcmp(argv[0],"passwd"))  bcpasswd(argc,argv);
   else if (!strcmp(argv[0],"reencrypt"))  bcreencrypt(argc,argv);
   else if (!strcmp(argv[0],"fsck"))  bcfsck(argc,argv);
   else {
      fprintf(stderr,"Unknown command %s\n",argv[0]);
      usage(stderr);
   }

   exit(0);
} /* end of main */

