/*******************************************************************
*
*      Copyright (c) 1994-1999 Jetico, Inc., Finland
*      All rights reserved.
*
*      File: bccreate.c
*      Revision:
*      Created:
*      Description: implementation of miscelleneous routines
*
*******************************************************************/
 


#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <unistd.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <string.h>
#include <signal.h>
#include <bc_types.h>
#include <bc_ioctl.h>
#include "alg.h"
#include <kgsha.h>
#include <kg_defs.h>
#include <fcntl.h>
#include "header.h"
#include <kblock.h>
#include "config.h"
#include "error.h"
#include "fstypes.h" 


#define MAX_DEV_PATH 16
#define MAX_DEVICES 16


char _device[MAX_DEV_PATH];

void SetSignalHandlers(void (*handler)(int)){

   signal(SIGINT,handler);
   signal(SIGQUIT,handler);
   signal(SIGTERM,handler);
   signal(SIGABRT,handler);

}



char* strtolower(char *string){
   int i;
   for (i=0;i<=strlen(string);i++)string[i]=tolower(string[i]);
   return string;
   
}



void msg(FILE *fd, char *msg_string){
    fprintf(fd,"%s\n",msg_string);
}



int is_valid_container(char *filename){

   struct HiddenSector  hs;
   int fd;

   
   if ((fd = open(filename, O_RDONLY)) < 0){
      perror (filename);
      return FALSE;
   }
   if (read(fd, &hs, sizeof(hs))!=sizeof(hs)){
      msg(stderr,"Can not read container header\n");
      close(fd);
      return FALSE;
   }

   close(fd);

   if (strncmp(hs.br.OEMid, "LOCOS94 ", sizeof(hs.br.OEMid))!=0){
      return FALSE;
   }
   
   if (strncmp(hs.br.volumeLabel, "CRYPTED_DSK", sizeof(hs.br.volumeLabel))!=0){
      return FALSE;
   }
  
   return TRUE;
 
};




unsigned long int string_to_long(char *string){

   char *endptr;
   unsigned long int result;
   
   result=strtoul(string,&endptr,0);
   if (string==endptr) return 0;
   if ((string[0]!='\0')&&(endptr[0]=='\0')) return result;
   if ((endptr[0]=='K' || endptr[0]=='k' )&&(endptr[1]=='\0')) {
      result *=1024;
      return result;
   };
   if ((endptr[0]=='M' || endptr[0]=='m')&&(endptr[1]=='\0')) {
      result *=1024*1024;
      return result;
   };
   return 0;

}


char* FindFreeDevice(){

   int i,n,rc;
   ALG_SERV Alg;
   
   for (i=0;i<MAX_DEVICES;i++){
      snprintf(_device,MAX_DEV_PATH,"/dev/bcrypt%d",i);
      Alg.bc_fd = open(_device, O_RDONLY);
      if (Alg.bc_fd >= 0){
	rc=bc_get_info(Alg,&n,&n);
	close(Alg.bc_fd);
	if(rc==0) return _device;
      }
   }
   
   return NULL;
}


char* FindMountedDevice(char *arg){

   struct stat buff1;
   int i,dev;

   if (lstat(arg,&buff1)!=0) return NULL;
   if (S_ISBLK(buff1.st_mode)) return arg;
   dev=buff1.st_dev;
   
   for (i=0;i<MAX_DEVICES;i++){
      snprintf(_device,MAX_DEV_PATH,"/dev/bcrypt%d",i);
      if (lstat(_device,&buff1)==0 && dev==buff1.st_rdev) return _device;
   }

   return NULL;
}


int IsOwner(int uid, char *arg){

   	struct stat buff;
	
 	if (lstat(arg,&buff)!=0) return -1;
	if (buff.st_uid==uid) return TRUE;
	return FALSE;
				
}	


int run_process(char *file, char **argv){
   
   int pid,status;
   
   if (file==NULL)return 1;
   pid=fork();
   if (pid==-1) return -1;
   if (pid==0){
      execve(file,argv,NULL);
      exit(127);
   }
   do {
      if (waitpid(pid,&status,0)==-1){
	 if (errno != EINTR) return -1;
      } else return status;
   } while(1);

}




int ask_y_n(char *string)
{
   printf(string);
   if (getchar() == 'y') return TRUE;
   return FALSE;
}




void progress (char *s, int p){
    printf (s,p);
}



char* getFStypeById(unsigned int id){
   
   if (id > MAX_ID)return NULL;
   return fstypes[id];
}

int   getIdByFStype(char *fstype){

   int i;
   
   for (i=0;i<MAX_ID;i++)if (strcmp(fstype,fstypes[i])==0) return i; 
   return MAX_ID;

}





void usage(FILE *fd){
   char *usage_string="Usage:\n\
bctool command [options] arguments\n\
\n\
commands:\n\
   new    - create container   (-a alg_name -s size [-d description] filename)\n\
   format - format container   ([-t fstype] filename)\n\
   mount  - mount container    ([-t fstype] [-r] filename mountpoint)\n\
   umount - unmount container  (mountpoint)\n\
   info   - get info on container (filename)\n\
   passwd - change container's password (filename)\n\
   fsck   - filesystem check (filename [-t fstype])\n\
   reencrypt - reencrypt container (-a alg_name filename)\n\
\n\
options:\n\
   -t fstype   - to specify filesystem type (default: msdos)\n\
   -a alg_name - to specify algorithm name\n\
   -s size     - to specify container size\n\
        example: -t xxx - in bytes; -t xxxK - in Kb; -t xxxM - in Mb\n\
   -d desc     - to specify description of container (up to 64 symbols)\n\
   -r          - to mount read only\n\
\n";
   
   fprintf(fd,usage_string);
   
}




